<?php

	/***************************************************************
	 *  Copyright notice
	 *
	 *  (c) 2016 Immowelt AG <support@immowelt.de>
	 *  All rights reserved
	 *
	 *  This script is part of the TYPO3 project. The TYPO3 project is
	 *  free software; you can redistribute it and/or modify
	 *  it under the terms of the GNU General Public License as published by
	 *  the Free Software Foundation; either version 2 of the License, or
	 *  (at your option) any later version.
	 *
	 *  The GNU General Public License can be found at
	 *  http://www.gnu.org/copyleft/gpl.html.
	 *
	 *  This script is distributed in the hope that it will be useful,
	 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
	 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 *  GNU General Public License for more details.
	 *
	 *  This copyright notice MUST APPEAR in all copies of the script!
	 ***************************************************************/

	namespace IWAG\IwImmo\Controller;

	use IWAG\IwImmo\Service\Exception\ServiceException;
	use IWAG\IwImmo\Utility\ImmoUtility;
	use TYPO3\CMS\Core\Utility\GeneralUtility;
	use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
	use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
	use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;

	if (!defined('TYPO3_MODE')) {
		die ('Access denied.');
	}

	/**
	 * Class BaseController
	 *
	 * @package IWAG\IwImmo\Controller
	 */
	class BaseController extends ActionController {

		/**
		 * @param ConfigurationManagerInterface $configurationManager
		 * @return void
		 */
		public function injectConfigurationManager(ConfigurationManagerInterface $configurationManager) {
			$this->configurationManager = $configurationManager;
			$this->settings = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);

			$this->settings = ImmoUtility::evaluateSettingsWithStdWrap($this->settings);

		}

		/**
		 * initialisiert die view mit dem template aus dem conf.
		 *
		 * @param ViewInterface $view
		 */
		protected function initializeView(ViewInterface $view)
		{
			parent::initializeView($view);

			$configuration = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

			if(isset($configuration['view']['layoutRootPath']) && $configuration['view']['layoutRootPath'] != "") {
				$this->view->setLayoutRootPath($configuration['view']['layoutRootPath']);
			}

			if(isset($configuration['view']['partialRootPath']) && $configuration['view']['partialRootPath'] != "") {
				$this->view->setPartialRootPath($configuration['view']['partialRootPath']);
			}

			if(isset($configuration['view']['templateRootPath']) && $configuration['view']['templateRootPath'] != "") {
				$this->view->setTemplateRootPath($configuration['view']['templateRootPath']);
			}
		}

		/**
		 * Alle ServiceExceptions (ausser ApiKeyMissingException) abfangen und zur ErrorArction weiterleiten
		 */
		protected function callActionMethod() {
			try {
				parent::callActionMethod();

			} catch (ServiceException $e) {
				if( get_class($e) != 'IWAG\IwImmo\Service\Exception\ApiKeyMissingException') {
					GeneralUtility::sysLog($e->getCode() . ': ' . $e->getMessage(), $this->extensionName, GeneralUtility::SYSLOG_SEVERITY_ERROR);
					$this->forward('error');
				}
				else {
					throw new $e;
				}
			}
		}

		/**
		 * @return string|void
		 */
		public function errorAction() {

		}

		/**
		 * @return string
		 */
		protected function getDisclaimer() {
			return '<div class="poweredby"><a href="http://www.immowelt.de" title="Immobilienportal Immowelt.de" rel="nofollow">Immobilien-Daten bereitgestellt von Immowelt.de</a> <img src="typo3conf/ext/iw_immo/Resources/Public/Images/powered-by-iw.jpg" alt="immowelt.de"></div><div class="clearfix"></div>';
		}
	}

	?>